@@ -1,104 +1,121 @@
-{-# LANGUAGE GeneralizedNewtypeDeriving, DeriveGeneric #-}
-module Aws.DynamoDb.Commands.Table(
-  -- * Commands
-    CreateTable(..)
-  , CreateTableResult(..)
-  , DescribeTable(..)
-  , DescribeTableResult(..)
-  , UpdateTable(..)
-  , UpdateTableResult(..)
-  , DeleteTable(..)
-  , DeleteTableResult(..)
-  , ListTables(..)
-  , ListTablesResult(..)
+{-# LANGUAGE DeriveGeneric              #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE TypeFamilies               #-}
 
-  -- * Data passed in the commands
-  , KeyAttributeType(..)
-  , KeyAttributeDefinition(..)
-  , KeySchema(..)
-  , Projection(..)
-  , LocalSecondaryIndex(..)
-  , LocalSecondaryIndexStatus(..)
-  , ProvisionedThroughput(..)
-  , ProvisionedThroughputStatus(..)
-  , GlobalSecondaryIndex(..)
-  , GlobalSecondaryIndexStatus(..)
-  , GlobalSecondaryIndexUpdate(..)
-  , TableDescription(..)
-) where
+module Aws.DynamoDb.Commands.Table
+    ( -- * Commands
+      CreateTable(..)
+    , createTable
+    , CreateTableResult(..)
+    , DescribeTable(..)
+    , DescribeTableResult(..)
+    , UpdateTable(..)
+    , UpdateTableResult(..)
+    , DeleteTable(..)
+    , DeleteTableResult(..)
+    , ListTables(..)
+    , ListTablesResult(..)
 
-import           Aws.Core
-import           Aws.DynamoDb.Core
+    -- * Data passed in the commands
+    , AttributeType(..)
+    , AttributeDefinition(..)
+    , KeySchema(..)
+    , Projection(..)
+    , LocalSecondaryIndex(..)
+    , LocalSecondaryIndexStatus(..)
+    , ProvisionedThroughput(..)
+    , ProvisionedThroughputStatus(..)
+    , GlobalSecondaryIndex(..)
+    , GlobalSecondaryIndexStatus(..)
+    , GlobalSecondaryIndexUpdate(..)
+    , TableDescription(..)
+    ) where
+
+-------------------------------------------------------------------------------
 import           Control.Applicative
-import           Data.Aeson ((.=), (.:), (.!=), (.:?))
-import qualified Data.Aeson as A
-import qualified Data.Aeson.Types as A
-import           Data.Char (toUpper)
+import           Data.Aeson            ((.!=), (.:), (.:?), (.=))
+import qualified Data.Aeson            as A
+import qualified Data.Aeson.Types      as A
+import           Data.Char             (toUpper)
+import qualified Data.HashMap.Strict   as M
+import qualified Data.Text             as T
 import           Data.Time
 import           Data.Time.Clock.POSIX
-import qualified Data.Text             as T
+import           Data.Typeable
 import qualified Data.Vector           as V
-import qualified Data.HashMap.Strict   as M
-import           GHC.Generics (Generic)
+import           GHC.Generics          (Generic)
+-------------------------------------------------------------------------------
+import           Aws.Core
+import           Aws.DynamoDb.Core
+-------------------------------------------------------------------------------
+
 
 capitalizeOpt :: A.Options
-capitalizeOpt = A.defaultOptions { A.fieldLabelModifier = \x -> case x of
-                                                                  (c:cs) -> toUpper c : cs
-                                                                  [] -> []
-                                 }
+capitalizeOpt = A.defaultOptions
+    { A.fieldLabelModifier = \x -> case x of
+                                     (c:cs) -> toUpper c : cs
+                                     [] -> []
+    }
 
 
 dropOpt :: Int -> A.Options
 dropOpt d = A.defaultOptions { A.fieldLabelModifier = drop d }
 
 
--- | The type of a key attribute that appears in the table key or as a key in one of the indices.
-data KeyAttributeType = AttrStringT | AttrNumberT | AttrBinaryT
-    deriving (Show, Eq, Enum, Bounded, Generic)
-instance A.ToJSON KeyAttributeType where
-    toJSON AttrStringT = A.String "S"
-    toJSON AttrNumberT = A.String "N"
-    toJSON AttrBinaryT = A.String "B"
-instance A.FromJSON KeyAttributeType where
+-- | The type of a key attribute that appears in the table key or as a
+-- key in one of the indices.
+data AttributeType = AttrString | AttrNumber | AttrBinary
+    deriving (Show, Read, Ord, Typeable, Eq, Enum, Bounded, Generic)
+
+instance A.ToJSON AttributeType where
+    toJSON AttrString = A.String "S"
+    toJSON AttrNumber = A.String "N"
+    toJSON AttrBinary = A.String "B"
+
+instance A.FromJSON AttributeType where
     parseJSON (A.String str) =
         case str of
-            "S" -> return AttrStringT
-            "N" -> return AttrNumberT
-            "B" -> return AttrBinaryT
+            "S" -> return AttrString
+            "N" -> return AttrNumber
+            "B" -> return AttrBinary
             _   -> fail $ "Invalid attribute type " ++ T.unpack str
     parseJSON _ = fail "Attribute type must be a string"
 
 -- | A key attribute that appears in the table key or as a key in one of the indices.
-data KeyAttributeDefinition
-    = KeyAttributeDefinition {
-        attributeName :: T.Text
-      , attributeType :: KeyAttributeType
-      }
-    deriving (Show, Generic)
-instance A.ToJSON KeyAttributeDefinition where
+data AttributeDefinition = AttributeDefinition {
+      attributeName :: T.Text
+    , attributeType :: AttributeType
+    } deriving (Eq,Read,Ord,Show,Typeable,Generic)
+
+instance A.ToJSON AttributeDefinition where
     toJSON = A.genericToJSON capitalizeOpt
-instance A.FromJSON KeyAttributeDefinition where
+
+instance A.FromJSON AttributeDefinition where
     parseJSON = A.genericParseJSON capitalizeOpt
 
 -- | The key schema can either be a hash of a single attribute name or a hash attribute name
 -- and a range attribute name.
-data KeySchema = KeyHashOnly T.Text
-               | KeyHashAndRange T.Text T.Text
-    deriving (Show)
+data KeySchema = HashOnly T.Text
+               | HashAndRange T.Text T.Text
+    deriving (Eq,Read,Show,Ord,Typeable,Generic)
+
+
 instance A.ToJSON KeySchema where
-    toJSON (KeyHashOnly attr) 
-        = A.Array $ V.fromList [ A.object [ "AttributeName" .= attr
-                                          , "KeyType" .= ("HASH" :: T.Text)
+    toJSON (HashOnly a)
+        = A.Array $ V.fromList [ A.object [ "AttributeName" .= a
+                                          , "KeyType" .= (A.String "HASH")
                                           ]
                                ]
-    toJSON (KeyHashAndRange hash range) 
+
+    toJSON (HashAndRange hash range)
         = A.Array $ V.fromList [ A.object [ "AttributeName" .= hash
-                                          , "KeyType" .= ("HASH" :: T.Text)
+                                          , "KeyType" .= (A.String "HASH")
                                           ]
                                , A.object [ "AttributeName" .= range
-                                          , "KeyType" .= ("RANGE" :: T.Text)
+                                          , "KeyType" .= (A.String "RANGE")
                                           ]
                                ]
+
 instance A.FromJSON KeySchema where
     parseJSON (A.Array v) =
         case V.length v of
@@ -106,7 +123,7 @@ instance A.FromJSON KeySchema where
                     kt <- obj .: "KeyType"
                     if kt /= ("HASH" :: T.Text)
                         then fail "With only one key, the type must be HASH"
-                        else KeyHashOnly <$> obj .: "AttributeName"
+                        else HashOnly <$> obj .: "AttributeName"
 
             2 -> do hash <- A.parseJSON (v V.! 0)
                     range <- A.parseJSON (v V.! 1)
@@ -114,8 +131,8 @@ instance A.FromJSON KeySchema where
                     rkt <- range .: "KeyType"
                     if hkt /= ("HASH" :: T.Text) || rkt /= ("RANGE" :: T.Text)
                         then fail "With two keys, one must be HASH and the other RANGE"
-                        else KeyHashAndRange <$> hash .: "AttributeName"
-                                             <*> range .: "AttributeName"
+                        else HashAndRange <$> hash .: "AttributeName"
+                                          <*> range .: "AttributeName"
             _ -> fail "Key schema must have one or two entries"
     parseJSON _ = fail "Key schema must be an array"
 
@@ -139,13 +156,14 @@ instance A.FromJSON Projection where
             "INCLUDE" -> ProjectInclude <$> o .: "NonKeyAttributes"
             _ -> fail "Invalid projection type"
     parseJSON _ = fail "Projection must be an object"
-      
--- | Describes a single local secondary index.  The KeySchema MUST share the same hash key attribute
--- as the parent table, only the range key can differ.
+
+-- | Describes a single local secondary index. The KeySchema MUST
+-- share the same hash key attribute as the parent table, only the
+-- range key can differ.
 data LocalSecondaryIndex
     = LocalSecondaryIndex {
-        localIndexName :: T.Text
-      , localKeySchema :: KeySchema
+        localIndexName  :: T.Text
+      , localKeySchema  :: KeySchema
       , localProjection :: Projection
       }
     deriving (Show, Generic)
@@ -157,11 +175,11 @@ instance A.FromJSON LocalSecondaryIndex where
 -- | This is returned by AWS to describe the local secondary index.
 data LocalSecondaryIndexStatus
     = LocalSecondaryIndexStatus {
-        locStatusIndexName :: T.Text
+        locStatusIndexName      :: T.Text
       , locStatusIndexSizeBytes :: Integer
-      , locStatusItemCount :: Integer
-      , locStatusKeySchema :: KeySchema
-      , locStatusProjection :: Projection
+      , locStatusItemCount      :: Integer
+      , locStatusKeySchema      :: KeySchema
+      , locStatusProjection     :: Projection
       }
     deriving (Show, Generic)
 instance A.FromJSON LocalSecondaryIndexStatus where
@@ -170,7 +188,7 @@ instance A.FromJSON LocalSecondaryIndexStatus where
 -- | The target provisioned throughput you are requesting for the table or global secondary index.
 data ProvisionedThroughput
     = ProvisionedThroughput {
-        readCapacityUnits :: Int
+        readCapacityUnits  :: Int
       , writeCapacityUnits :: Int
       }
     deriving (Show, Generic)
@@ -182,11 +200,11 @@ instance A.FromJSON ProvisionedThroughput where
 -- | This is returned by AWS as the status of the throughput for a table or global secondary index.
 data ProvisionedThroughputStatus
     = ProvisionedThroughputStatus {
-        statusLastDecreaseDateTime :: UTCTime
-      , statusLastIncreaseDateTime :: UTCTime
+        statusLastDecreaseDateTime   :: UTCTime
+      , statusLastIncreaseDateTime   :: UTCTime
       , statusNumberOfDecreasesToday :: Int
-      , statusReadCapacityUnits :: Int
-      , statusWriteCapacityUnits :: Int
+      , statusReadCapacityUnits      :: Int
+      , statusWriteCapacityUnits     :: Int
       }
     deriving (Show, Generic)
 instance A.FromJSON ProvisionedThroughputStatus where
@@ -201,9 +219,9 @@ instance A.FromJSON ProvisionedThroughputStatus where
 -- | Describes a global secondary index.
 data GlobalSecondaryIndex
     = GlobalSecondaryIndex {
-        globalIndexName :: T.Text
-      , globalKeySchema :: KeySchema
-      , globalProjection :: Projection
+        globalIndexName             :: T.Text
+      , globalKeySchema             :: KeySchema
+      , globalProjection            :: Projection
       , globalProvisionedThroughput :: ProvisionedThroughput
       }
     deriving (Show, Generic)
@@ -215,44 +233,46 @@ instance A.FromJSON GlobalSecondaryIndex where
 -- | This is returned by AWS to describe the status of a global secondary index.
 data GlobalSecondaryIndexStatus
     = GlobalSecondaryIndexStatus {
-        gStatusIndexName :: T.Text
-      , gStatusIndexSizeBytes :: Integer
-      , gStatusIndexStatus :: T.Text
-      , gStatusItemCount :: Integer
-      , gStatusKeySchema :: KeySchema
-      , gStatusProjection :: Projection
+        gStatusIndexName             :: T.Text
+      , gStatusIndexSizeBytes        :: Integer
+      , gStatusIndexStatus           :: T.Text
+      , gStatusItemCount             :: Integer
+      , gStatusKeySchema             :: KeySchema
+      , gStatusProjection            :: Projection
       , gStatusProvisionedThroughput :: ProvisionedThroughputStatus
       }
     deriving (Show, Generic)
 instance A.FromJSON GlobalSecondaryIndexStatus where
     parseJSON = A.genericParseJSON $ dropOpt 7
 
--- | This is used to request a change in the provisioned throughput of a global secondary index as
--- part of an 'UpdateTable' operation.
+-- | This is used to request a change in the provisioned throughput of
+-- a global secondary index as part of an 'UpdateTable' operation.
 data GlobalSecondaryIndexUpdate
     = GlobalSecondaryIndexUpdate {
-        gUpdateIndexName :: T.Text
+        gUpdateIndexName             :: T.Text
       , gUpdateProvisionedThroughput :: ProvisionedThroughput
       }
     deriving (Show, Generic)
 instance A.ToJSON GlobalSecondaryIndexUpdate where
     toJSON gi = A.object ["Update" .= A.genericToJSON (dropOpt 7) gi]
 
--- | This describes the table and is the return value from AWS for all the table-related commands.
+-- | This describes the table and is the return value from AWS for all
+-- the table-related commands.
 data TableDescription
     = TableDescription {
-        rTableName :: T.Text
-      , rTableSizeBytes :: Integer
-      , rTableStatus :: T.Text -- ^ one of CREATING, UPDATING, DELETING, ACTIVE
-      , rCreationDateTime :: UTCTime
-      , rItemCount :: Integer
-      , rAttributeDefinitions :: [KeyAttributeDefinition]
-      , rKeySchema :: KeySchema
-      , rProvisionedThroughput :: ProvisionedThroughputStatus
-      , rLocalSecondaryIndexes :: [LocalSecondaryIndexStatus]
+        rTableName              :: T.Text
+      , rTableSizeBytes         :: Integer
+      , rTableStatus            :: T.Text -- ^ one of CREATING, UPDATING, DELETING, ACTIVE
+      , rCreationDateTime       :: Maybe UTCTime
+      , rItemCount              :: Integer
+      , rAttributeDefinitions   :: [AttributeDefinition]
+      , rKeySchema              :: Maybe KeySchema
+      , rProvisionedThroughput  :: ProvisionedThroughputStatus
+      , rLocalSecondaryIndexes  :: [LocalSecondaryIndexStatus]
       , rGlobalSecondaryIndexes :: [GlobalSecondaryIndexStatus]
       }
     deriving (Show, Generic)
+
 instance A.FromJSON TableDescription where
     parseJSON = A.withObject "Table must be an object" $ \o -> do
         t <- case (M.lookup "Table" o, M.lookup "TableDescription" o) of
@@ -262,18 +282,18 @@ instance A.FromJSON TableDescription where
         TableDescription <$> t .: "TableName"
                          <*> t .: "TableSizeBytes"
                          <*> t .: "TableStatus"
-                         <*> (posixSecondsToUTCTime . fromInteger <$> t .: "CreationDateTime")
+                         <*> (fmap (posixSecondsToUTCTime . fromInteger) <$> t .:? "CreationDateTime")
                          <*> t .: "ItemCount"
-                         <*> t .: "AttributeDefinitions"
-                         <*> t .: "KeySchema"
+                         <*> t .:? "AttributeDefinitions" .!= []
+                         <*> t .:? "KeySchema"
                          <*> t .: "ProvisionedThroughput"
                          <*> t .:? "LocalSecondaryIndexes" .!= []
                          <*> t .:? "GlobalSecondaryIndexes" .!= []
-             
+
 {- Can't derive these instances onto the return values
 instance ResponseConsumer r TableDescription where
     type ResponseMetadata TableDescription = DyMetadata
-    responseConsumer _ _ = dyResponseConsumer
+    responseConsumer _ _ = ddbResponseConsumer
 instance AsMemoryResponse TableDescription where
     type MemoryResponse TableDescription = TableDescription
     loadToMemory = return
@@ -283,16 +303,24 @@ instance AsMemoryResponse TableDescription where
 --- Commands
 -------------------------------------------------------------------------------
 
-data CreateTable
-    = CreateTable {
-        createTableName :: T.Text
-      , createAttributeDefinitions :: [KeyAttributeDefinition] -- ^ only attributes appearing in a key must be listed here
-      , createKeySchema :: KeySchema
-      , createProvisionedThroughput :: ProvisionedThroughput
-      , createLocalSecondaryIndexes :: [LocalSecondaryIndex] -- ^ at most 5 local secondary indices are allowed
-      , createGlobalSecondaryIndexes :: [GlobalSecondaryIndex]
-      }
-    deriving (Show, Generic)
+data CreateTable = CreateTable {
+      createTableName              :: T.Text
+    , createAttributeDefinitions   :: [AttributeDefinition]
+    -- ^ only attributes appearing in a key must be listed here
+    , createKeySchema              :: KeySchema
+    , createProvisionedThroughput  :: ProvisionedThroughput
+    , createLocalSecondaryIndexes  :: [LocalSecondaryIndex]
+    -- ^ at most 5 local secondary indices are allowed
+    , createGlobalSecondaryIndexes :: [GlobalSecondaryIndex]
+    } deriving (Show, Generic)
+
+createTable :: T.Text -- ^ Table name
+            -> [AttributeDefinition]
+            -> KeySchema
+            -> ProvisionedThroughput
+            -> CreateTable
+createTable tn ad ks p = CreateTable tn ad ks p [] []
+
 instance A.ToJSON CreateTable where
     toJSON ct = A.object $ m ++ lindex ++ gindex
         where
@@ -308,20 +336,22 @@ instance A.ToJSON CreateTable where
             gindex = if null (createGlobalSecondaryIndexes ct)
                         then []
                         else [ "GlobalSecondaryIndexes" .= createGlobalSecondaryIndexes ct ]
+
 --instance A.ToJSON CreateTable where
 --    toJSON = A.genericToJSON $ dropOpt 6
-        
--- | ServiceConfiguration: 'DyConfiguration'
+
+
+-- | ServiceConfiguration: 'DdbConfiguration'
 instance SignQuery CreateTable where
-    type ServiceConfiguration CreateTable = DyConfiguration
-    signQuery = dySignQuery "CreateTable"
+    type ServiceConfiguration CreateTable = DdbConfiguration
+    signQuery = ddbSignQuery "CreateTable"
 
 newtype CreateTableResult = CreateTableResult { ctStatus :: TableDescription }
     deriving (Show, A.FromJSON)
 -- ResponseConsumer and AsMemoryResponse can't be derived
 instance ResponseConsumer r CreateTableResult where
-    type ResponseMetadata CreateTableResult = DyMetadata
-    responseConsumer _ _ = dyResponseConsumer
+    type ResponseMetadata CreateTableResult = DdbResponse
+    responseConsumer _ = ddbResponseConsumer
 instance AsMemoryResponse CreateTableResult where
     type MemoryResponse CreateTableResult = TableDescription
     loadToMemory = return . ctStatus
@@ -330,23 +360,23 @@ instance Transaction CreateTable CreateTableResult
 
 data DescribeTable
     = DescribeTable {
-        dTableName :: T.Text 
+        dTableName :: T.Text
       }
     deriving (Show, Generic)
 instance A.ToJSON DescribeTable where
     toJSON = A.genericToJSON $ dropOpt 1
 
--- | ServiceConfiguration: 'DyConfiguration'
+-- | ServiceConfiguration: 'DdbConfiguration'
 instance SignQuery DescribeTable where
-    type ServiceConfiguration DescribeTable = DyConfiguration
-    signQuery = dySignQuery "DescribeTable"
+    type ServiceConfiguration DescribeTable = DdbConfiguration
+    signQuery = ddbSignQuery "DescribeTable"
 
 newtype DescribeTableResult = DescribeTableResult { dtStatus :: TableDescription }
     deriving (Show, A.FromJSON)
 -- ResponseConsumer can't be derived
 instance ResponseConsumer r DescribeTableResult where
-    type ResponseMetadata DescribeTableResult = DyMetadata
-    responseConsumer _ _ = dyResponseConsumer
+    type ResponseMetadata DescribeTableResult = DdbResponse
+    responseConsumer _ = ddbResponseConsumer
 instance AsMemoryResponse DescribeTableResult where
     type MemoryResponse DescribeTableResult = TableDescription
     loadToMemory = return . dtStatus
@@ -355,25 +385,30 @@ instance Transaction DescribeTable DescribeTableResult
 
 data UpdateTable
     = UpdateTable {
-        updateTableName :: T.Text
-      , updateProvisionedThroughput :: ProvisionedThroughput
+        updateTableName                   :: T.Text
+      , updateProvisionedThroughput       :: ProvisionedThroughput
       , updateGlobalSecondaryIndexUpdates :: [GlobalSecondaryIndexUpdate]
       }
     deriving (Show, Generic)
 instance A.ToJSON UpdateTable where
-    toJSON = A.genericToJSON $ dropOpt 6
-    
--- | ServiceConfiguration: 'DyConfiguration'
+    toJSON a = A.object
+        $ "TableName" .= updateTableName a
+        : "ProvisionedThroughput" .= updateProvisionedThroughput a
+        : case updateGlobalSecondaryIndexUpdates a of
+            [] -> []
+            l -> [ "GlobalSecondaryIndexUpdates" .= l ]
+
+-- | ServiceConfiguration: 'DdbConfiguration'
 instance SignQuery UpdateTable where
-    type ServiceConfiguration UpdateTable = DyConfiguration
-    signQuery = dySignQuery "UpdateTable"
+    type ServiceConfiguration UpdateTable = DdbConfiguration
+    signQuery = ddbSignQuery "UpdateTable"
 
 newtype UpdateTableResult = UpdateTableResult { uStatus :: TableDescription }
     deriving (Show, A.FromJSON)
 -- ResponseConsumer can't be derived
 instance ResponseConsumer r UpdateTableResult where
-    type ResponseMetadata UpdateTableResult = DyMetadata
-    responseConsumer _ _ = dyResponseConsumer
+    type ResponseMetadata UpdateTableResult = DdbResponse
+    responseConsumer _ = ddbResponseConsumer
 instance AsMemoryResponse UpdateTableResult where
     type MemoryResponse UpdateTableResult = TableDescription
     loadToMemory = return . uStatus
@@ -388,17 +423,17 @@ data DeleteTable
 instance A.ToJSON DeleteTable where
     toJSON = A.genericToJSON $ dropOpt 6
 
--- | ServiceConfiguration: 'DyConfiguration'
+-- | ServiceConfiguration: 'DdbConfiguration'
 instance SignQuery DeleteTable where
-    type ServiceConfiguration DeleteTable = DyConfiguration
-    signQuery = dySignQuery "DeleteTable"
+    type ServiceConfiguration DeleteTable = DdbConfiguration
+    signQuery = ddbSignQuery "DeleteTable"
 
 newtype DeleteTableResult = DeleteTableResult { dStatus :: TableDescription }
     deriving (Show, A.FromJSON)
 -- ResponseConsumer can't be derived
 instance ResponseConsumer r DeleteTableResult where
-    type ResponseMetadata DeleteTableResult = DyMetadata
-    responseConsumer _ _ = dyResponseConsumer
+    type ResponseMetadata DeleteTableResult = DdbResponse
+    responseConsumer _ = ddbResponseConsumer
 instance AsMemoryResponse DeleteTableResult where
     type MemoryResponse DeleteTableResult = TableDescription
     loadToMemory = return . dStatus
@@ -410,23 +445,23 @@ data ListTables = ListTables
     deriving (Show)
 instance A.ToJSON ListTables where
     toJSON _ = A.object []
--- | ServiceConfiguration: 'DyConfiguration'
+-- | ServiceConfiguration: 'DdbConfiguration'
 instance SignQuery ListTables where
-    type ServiceConfiguration ListTables = DyConfiguration
-    signQuery = dySignQuery "ListTables"
+    type ServiceConfiguration ListTables = DdbConfiguration
+    signQuery = ddbSignQuery "ListTables"
 
 newtype ListTablesResult
     = ListTablesResult {
-        tableNames :: [T.Text] 
+        tableNames :: [T.Text]
       }
     deriving (Show, Generic)
 instance A.FromJSON ListTablesResult where
     parseJSON = A.genericParseJSON capitalizeOpt
 instance ResponseConsumer r ListTablesResult where
-    type ResponseMetadata ListTablesResult = DyMetadata
-    responseConsumer _ _ = dyResponseConsumer
+    type ResponseMetadata ListTablesResult = DdbResponse
+    responseConsumer _ = ddbResponseConsumer
 instance AsMemoryResponse ListTablesResult where
-    type MemoryResponse ListTablesResult = [T.Text] 
+    type MemoryResponse ListTablesResult = [T.Text]
     loadToMemory = return . tableNames
 
 instance Transaction ListTables ListTablesResult
