@@ -1,23 +1,29 @@
 module Aws.S3.Commands
 (
   module Aws.S3.Commands.CopyObject
+, module Aws.S3.Commands.DeleteBucket
 , module Aws.S3.Commands.DeleteObject
 , module Aws.S3.Commands.DeleteObjects
 , module Aws.S3.Commands.GetBucket
+, module Aws.S3.Commands.GetBucketLocation
 , module Aws.S3.Commands.GetObject
 , module Aws.S3.Commands.GetService
 , module Aws.S3.Commands.HeadObject
 , module Aws.S3.Commands.PutBucket
 , module Aws.S3.Commands.PutObject
+, module Aws.S3.Commands.Multipart
 )
 where
 
 import Aws.S3.Commands.CopyObject
+import Aws.S3.Commands.DeleteBucket
 import Aws.S3.Commands.DeleteObject
 import Aws.S3.Commands.DeleteObjects
 import Aws.S3.Commands.GetBucket
+import Aws.S3.Commands.GetBucketLocation
 import Aws.S3.Commands.GetObject
 import Aws.S3.Commands.GetService
 import Aws.S3.Commands.HeadObject
 import Aws.S3.Commands.PutBucket
 import Aws.S3.Commands.PutObject
+import Aws.S3.Commands.Multipart
