@@ -12,7 +12,6 @@ import           Data.ByteString.Char8 ({- IsString -})
 import           Data.Conduit
 import qualified Data.Conduit.List     as CL
 import           Data.Maybe
-import           Data.Monoid
 import           Text.XML.Cursor       (($/))
 import qualified Data.ByteString.Char8 as B8
 import qualified Data.ByteString.Lazy  as BL
@@ -131,12 +130,13 @@ data UploadPart = UploadPart {
   , upContentMD5 :: Maybe (Digest MD5)
   , upServerSideEncryption :: Maybe ServerSideEncryption
   , upRequestBody  :: HTTP.RequestBody
+  , upExpect100Continue :: Bool -- ^ Note: Requires http-client >= 0.4.10
 }
 
 uploadPart :: Bucket -> T.Text -> Integer -> T.Text -> HTTP.RequestBody -> UploadPart
 uploadPart bucket obj p i body =
   UploadPart obj bucket p i
-  Nothing Nothing Nothing body
+  Nothing Nothing Nothing body False
 
 data UploadPartResponse
   = UploadPartResponse {
@@ -162,7 +162,11 @@ instance SignQuery UploadPart where
                                , s3QAmzHeaders = map (second T.encodeUtf8) $ catMaybes [
                                    ("x-amz-server-side-encryption",) <$> writeServerSideEncryption <$> upServerSideEncryption
                                  ]
-                               , s3QOtherHeaders = []
+                               , s3QOtherHeaders = catMaybes [
+                                    if upExpect100Continue
+                                        then Just ("Expect", "100-continue")
+                                        else Nothing
+                                 ]
                                , s3QRequestBody = Just upRequestBody
                                }
 
@@ -380,25 +384,21 @@ putConduit cfg s3cfg mgr bucket object uploadId = loop 1
         Nothing -> return ()
 
 chunkedConduit :: (MonadResource m) => Integer -> Conduit B8.ByteString m BL.ByteString
-chunkedConduit size = do
-  loop 0 ""
+chunkedConduit size = loop 0 []
   where
-    loop :: MonadResource m => Int -> BL.ByteString -> Conduit B8.ByteString m BL.ByteString
-    loop cnt str = do
-      line' <- await
-      case line' of 
-        Nothing -> do
-          yield str
-          return ()
-        Just line -> do
-          let len = B8.length line+cnt
-          let newStr = str <> BL.fromStrict line
-          if len >= (fromIntegral size)
-            then do
-            yield newStr
-            loop 0 ""
-            else
-            loop len newStr
+    loop :: Monad m => Integer -> [B8.ByteString] -> Conduit B8.ByteString m BL.ByteString
+    loop cnt str = await >>= maybe (yieldChunk str) go
+      where
+        go :: Monad m => B8.ByteString -> Conduit B8.ByteString m BL.ByteString
+        go line
+          | size <= len = yieldChunk newStr >> loop 0 []
+          | otherwise   = loop len newStr
+          where
+            len = fromIntegral (B8.length line) + cnt
+            newStr = line:str
+
+    yieldChunk :: Monad m => [B8.ByteString] -> Conduit i m BL.ByteString
+    yieldChunk = yield . BL.fromChunks . reverse
 
 multipartUpload ::
   Configuration
@@ -417,6 +417,16 @@ multipartUpload cfg s3cfg mgr bucket object src chunkSize = do
            $$ CL.consume
   liftIO $ sendEtag cfg s3cfg mgr bucket object uploadId etags
 
+multipartUploadSink :: MonadResource m
+  => Configuration
+  -> S3Configuration NormalQuery
+  -> HTTP.Manager
+  -> T.Text    -- ^ Bucket name
+  -> T.Text    -- ^ Object name
+  -> Integer   -- ^ chunkSize (minimum: 5MB)
+  -> Sink B8.ByteString m ()
+multipartUploadSink cfg s3cfg = multipartUploadSinkWithInitiator cfg s3cfg postInitiateMultipartUpload
+
 multipartUploadWithInitiator ::
   Configuration
   -> S3Configuration NormalQuery
@@ -434,3 +444,19 @@ multipartUploadWithInitiator cfg s3cfg initiator mgr bucket object src chunkSize
            $= putConduit cfg s3cfg mgr bucket object uploadId
            $$ CL.consume
   liftIO $ sendEtag cfg s3cfg mgr bucket object uploadId etags
+
+multipartUploadSinkWithInitiator :: MonadResource m
+  => Configuration
+  -> S3Configuration NormalQuery
+  -> (Bucket -> T.Text -> InitiateMultipartUpload) -- ^ Initiator
+  -> HTTP.Manager
+  -> T.Text    -- ^ Bucket name
+  -> T.Text    -- ^ Object name
+  -> Integer   -- ^ chunkSize (minimum: 5MB)
+  -> Sink B8.ByteString m ()
+multipartUploadSinkWithInitiator cfg s3cfg initiator mgr bucket object chunkSize = do
+  uploadId <- liftIO $ imurUploadId <$> memoryAws cfg s3cfg mgr (initiator bucket object)
+  etags <- chunkedConduit chunkSize
+           $= putConduit cfg s3cfg mgr bucket object uploadId
+           $= CL.consume
+  liftIO $ sendEtag cfg s3cfg mgr bucket object uploadId etags
